# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import test
from django.core.files.base import ContentFile
from django_selectel_storage.storage import SelectelStorage
from django.utils import six
import uuid


LAZY_FOX = 'The *quick* brown fox jumps over the lazy dog'
EMPTY_GIF = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\xf0\x01\x00' \
            b'\xff\xff\xff\x00\x00\x00\x21\xf9\x04\x01\x0a\x00\x00' \
            b'\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02' \
            b'\x44\x01\x00\x3b'


class ExistingFile(object):
    def __init__(self, test_case, fn, cnt, file_class=ContentFile):
        self.test = test_case
        self.filename = self.test.session_id + '_' + fn
        self.content = file_class(cnt)

    def __enter__(self):
        self.test.storage.save(self.filename, self.content)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.test.storage.delete(self.filename)


class TestBase(test.TestCase):
    """
    Base test class
    """

    def setUp(self):
        self.storage = SelectelStorage()
        self.session_id = uuid.uuid4().hex

    def existing_file(self, filename, content=''):
        return ExistingFile(self, filename, content)


class TestOverrideSettings(object):
    """
    Test cases for setting overrides
    """
    setting = ''
    method = ''
    value = None

    def test_override_value(self):
        value = self.value or 'new value'

        with self.settings(**{self.setting: value}):
            self.assertEquals(first=value,
                              second=getattr(self.storage, self.method)())


class TestListDirMethod(TestBase):
    """
    Tests for `listdir` storage method
    """

    def test_listdir_empty(self):
        # Create unique directory to avoid race conditions during
        # concurrent test running
        self.storage.container.storage.session.put(
            url='{storage}/{container}{path}'.format(
                storage=self.storage.container.storage.auth.storage,
                container=self.storage.container.name,
                path=self.session_id
            ),
            headers={
                'Content-Type': 'application/directory'
            })
        self.assertEquals(
            first=([], []),
            second=self.storage.listdir('/' + self.session_id)
        )

    def test_listdir_non_empty(self):
        with self.existing_file('file.img') as f:
            dirs, files = self.storage.listdir('/')
            self.assertIn('/' + f.filename, files)


class TestExistsMethod(TestBase):
    """
    Tests for `exists` storage method
    """
    def test_exists_not(self):
        filename = self.session_id + '_non_exists.txt'
        self.assertFalse(self.storage.exists(filename))

    def test_exists_yes(self):
        with self.existing_file('file.img') as f:
            self.assertTrue(self.storage.exists(f.filename))


class TestDeleteMethod(TestBase):
    """
    Tests for `exists` storage method
    """
    def test_delete_non_exists(self):
        self.storage.delete(self.session_id + '_non_exists.txt')

    def test_delete_exists(self):
        with self.existing_file('exists.txt') as f:
            self.assertTrue(self.storage.exists(f.filename))
            self.storage.delete(f.filename)
            self.assertFalse(self.storage.exists(f.filename))


class TestUrlMethod(TestBase):
    """
    Tests for `url` storage method
    """
    def test_url_with_container_default_name(self):
        self.assertEquals(
            self.storage.get_base_url() + '/index.html',
            self.storage.url('index.html')
        )

    def test_container_custom_name(self):
        url = 'https://selectel.com'
        with self.settings(SELECTEL_CONTAINER_URL=url):
            storage = SelectelStorage()
            self.assertEquals(url, storage.get_base_url())


class TestSizeMethod(TestBase):
    """
    Tests for `url` storage method
    """
    def test_size_for_non_existing_file(self):
        def raise_exception():
            self.storage.size(self.session_id + '_non_exists.txt')
        self.assertRaises(IOError, raise_exception)

    def test_zero_size_file(self):
        with self.existing_file('zero_size.txt') as f:
            self.assertEquals(0, self.storage.size(f.filename))

    def test_size_binary_file(self):

        with self.existing_file('empty.gif', content=EMPTY_GIF) as f:
            self.assertEquals(len(EMPTY_GIF), self.storage.size(f.filename))

    def test_size_text_file(self):
        if six.PY3:
            self.skipTest('Does not work in py3')

        with self.existing_file('README.md', content=LAZY_FOX) as f:
            self.assertEquals(len(LAZY_FOX), self.storage.size(f.filename))


class TestOpenMethod(TestBase):
    """
    Tests for `_open` storage method
    """
    def test_get_binary_file(self):
        with self.existing_file('empty.gif', content=EMPTY_GIF) as f:
            content = self.storage._open(f.filename, 'rb')
            self.assertEquals(first=EMPTY_GIF,
                              second=content.read())

    def test_get_text_mode(self):
        if six.PY3:
            self.skipTest('Does not work in py3')

        with self.existing_file('file.txt', content=LAZY_FOX) as f:
            content = self.storage._open(f.filename, 'r')
            self.assertEquals(first=LAZY_FOX,
                              second=content.read())


class TestGetAuthMethod(TestOverrideSettings, TestBase):
    """
    Tests for `get_auth` storage method
    """
    setting = 'SELECTEL_USERNAME'
    method = 'get_auth'


class TestGetKeyMethod(TestOverrideSettings, TestBase):
    """
    Tests for `get_key` storage method
    """
    setting = 'SELECTEL_PASSWORD'
    method = 'get_key'


class TestGetContainerNameMethod(TestOverrideSettings, TestBase):
    """
    Tests for `get_container_method` storage method
    """
    setting = 'SELECTEL_CONTAINER_NAME'
    method = 'get_container_name'


class TestGetContainerUrlMethod(TestOverrideSettings, TestBase):
    """
    Tests for `get_container_url` storage method
    """
    setting = 'SELECTEL_CONTAINER_URL'
    method = 'get_container_url'


class TestGetRequestsAdapterMethod(TestBase):
    """
    Tests for `get_requests_adapter` storage method
    """


class TestGetNameMethod(TestBase):
    """
    Tests for `_name` internal storage method
    """
    def test_add_leading_slash_if_not(self):
        self.assertEquals('/index.html', self.storage._name('index.html'))

    def test_not_add_leading_slash_if_not_need(self):
        self.assertEquals('/index.html', self.storage._name('/index.html'))

    def test_strip_excess_leading_slashes(self):
        self.assertEquals('/index.html', self.storage._name('/////index.html'))

    # def get_auth(self, **kwargs):
    # def get_key(self, **kwargs):
    # def get_container_name(self, **kwargs):
    # def get_container_url(self, **kwargs):
    # def get_requests_adapter(self):
    # def mount_requests_adapter(self, prefix, adapter):
    # *def get_base_url(self):

    # def _name(self, name):
    # def _open(self, name, mode='rb'):
    # def _save(self, name, content):



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.core.files.storage import Storage as DjangoStorage
from django.core.files.base import ContentFile
from django.conf import settings
import os
import selectel
import requests


MAX_RETRIES = 3
POOL_CONNECTIONS = 50
POOL_MAXSIZE = 50


def setting(name, default=None):
    return getattr(settings, name, default)


class SelectelStorage(DjangoStorage):
    def __init__(self, **kwargs):
        self.container = selectel.storage.Container(
            auth=self.get_auth(**kwargs),
            key=self.get_key(**kwargs),
            name=self.get_container_name(**kwargs))
        self.setup_requests_adapter(**kwargs)

    def get_auth(self, **kwargs):
        return setting('SELECTEL_USERNAME')

    def get_key(self, **kwargs):
        return setting('SELECTEL_PASSWORD')

    def get_container_name(self, **kwargs):
        return setting('SELECTEL_CONTAINER_NAME')

    def get_container_url(self, **kwargs):
        return setting('SELECTEL_CONTAINER_URL')

    def get_requests_adapter(self, **kwargs):
        return requests.adapters.HTTPAdapter(
            max_retries=setting('SELECTEL_MAX_RETRIES',
                                MAX_RETRIES),
            pool_connections=setting('SELECTEL_POOL_CONNECTIONS',
                                     POOL_CONNECTIONS),
            pool_maxsize=setting('SELECTEL_POOL_MAXSIZE',
                                 POOL_MAXSIZE))

    def setup_requests_adapter(self, **kwargs):
        adapter = self.get_requests_adapter(**kwargs)
        self.mount_requests_adapter('http://', adapter)
        self.mount_requests_adapter('https://', adapter)

    def mount_requests_adapter(self, prefix, adapter):
        self.container.storage.session.mount(prefix, adapter)

    def get_base_url(self):
        base_url = self.get_container_url()
        if base_url:
            return base_url
        else:
            return '{netloc}/{container}'.format(
                netloc=self.container.storage.auth.storage,
                container=self.get_container_name()
            )

    def _name(self, name):
        return '/' + name.lstrip('/')

    def _open(self, name, mode='rb'):
        return ContentFile(self.container.get(self._name(name)))

    def _save(self, name, content):
        # if six.PY3:
        #     self.container.put_stream_py3(self._name(name), content)
        # else:
        self.container.put_stream(self._name(name), content)
        return name

    def delete(self, name):
        self.container.remove(self._name(name), force=True)

    def exists(self, name):
        try:
            self.container.info(self._name(name))
            return True
        except selectel.storage.requests.exceptions.HTTPError:
            return False

    def listdir(self, path):
        return (
            [],
            list(self.container.list(self._name(path)).keys())
        )

    def size(self, name):
        try:
            return self.container.info(self._name(name))['content-length']
        except requests.exceptions.HTTPError:
            raise IOError('Unable get size for %s' % name)

    def url(self, name):
        return os.path.join(self.get_base_url().rstrip('/'),  name.lstrip('/'))


class SelectelStaticStorage(SelectelStorage):
    container_name = setting('SELECTEL_STATIC_CONTAINER_NAME')
    base_url = setting('SELECTEL_STATIC_BASE_URL')



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from setuptools import setup, find_packages
import os


ROOT_PACKAGE = 'django-selectel-storage'
DIR = os.path.dirname(__file__)
VERSION = '0.1'
DESCRIPTION = ('This application allows you easily save media and static '
               'files into selectel cloud storage')


def long_description():
    """
    Returns package long description from README
    """
    def read(what):
        with open(os.path.join(DIR, '%s.rst' % what)) as fp:
            return fp.read()
    return '{README}\n'.format(README=read('README'))


def version():
    """
    Returns package version for package building
    """
    return VERSION


if __name__ == '__main__':
    setup(name=ROOT_PACKAGE,
          description=DESCRIPTION,
          author='Mikhail Porokhovnichenko',
          author_email='marazmiki@gmail.com',
          version=version(),
          long_description=long_description(),
          packages=find_packages(),
          include_package_data=True,
          test_suite='tests.main',
          install_requires=['requests', 'selectel-api'],
          tests_require=['requests', 'selectel-api'],
          zip_safe=False,
          classifiers=[
              'Environment :: Web Environment',
              'Programming Language :: Python',
              'Framework :: Django'
          ],
    )
